<?php

namespace Drupal\style_selector\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Html;
use Drupal\Component\Utility\Xss;

/**
 * Class ModuleConfigForm.
 *
 * Configuration settings for Style Selector module.
 */
class ModuleConfigForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'style_selector.settings';

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

    /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'style_selector_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    $form['libraries'] = [
      '#type' => 'details',
      '#title' => 'Libraries',
      '#open' => TRUE,
    ];

    $form['libraries']['shared_libraries'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Shared Libraries'),
      '#rows' => 3,
      '#default_value' => $this->settingStringFromConfigData($config->get('shared_libraries'), "\n"),
      '#description' => $this->t(
        'One or more libraries (one per line) that will be loaded <strong>BOTH</strong> when a Style Selector CSS Class '
        . 'field is displayed with the CSS Class widget (e.g. in the admin UI) <strong>AND</strong> when a Style Selector field is rendered '
        . 'with the CSS Class formatter.'
        . '<br/><strong>Example</strong>: <code>my_theme/background_styles</code>'
      ),
    ];

    $form['libraries']['theme_libraries'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Theme-only Libraries'),
      '#rows' => 3,
      '#default_value' => $this->settingStringFromConfigData($config->get('theme_libraries'), "\n"),
      '#description' => $this->t(
        'One or more libraries (one per line) that will be loaded <strong>ONLY</strong> when a Style Selector field '
        . 'is rendered with the CSS Class formatter (i.e. in the <strong>user-facing theme</strong>).'
        . '<br/><strong>Example</strong>: <code>my_theme/background_support_styles</code><br/>'
      ),
    ];

    $form['libraries']['admin_libraries'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Admin-only Libraries'),
      '#rows' => 3,
      '#default_value' => $this->settingStringFromConfigData($config->get('admin_libraries'), "\n"),
      '#description' => $this->t(
        'One or more libraries (one per line) that will be loaded <strong>ONLY</strong> when a Style Selector CSS Class '
        . 'field is displayed with the CSS Class widget (i.e. in the <strong>admin UI</strong>).'
        . '<br/><strong>Example</strong>: <code>my_theme/style_selector_admin_styles</code>'
      ),
    ];

    $form['extra'] = [
      '#type' => 'details',
      '#title' => 'Extra class names',
      '#open' => TRUE,
    ];

    $form['extra']['css_classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS Class Field Classes'),
      '#default_value' => $this->settingStringFromConfigData($config->get('extra_css_classes')),
      '#description' => $this->t(
        'A space-separated list of classes to include when a CSS Class field is rendered with the CSS Class Formatter.',
      ),
    ];

    $form['extra']['color_classes'] = [
      '#type' => 'textfield',
      '#title' => $this->t('CSS Color Field Classes'),
      '#default_value' => $this->settingStringFromConfigData($config->get('extra_color_classes')),
      '#description' => $this->t(
        'A space-separated list of classes to include when a CSS Color field is rendered with the CSS Color Formatter.',
      ),
    ];

    // Sample Style Selector.
    // $form['sample_style'] = [
    //   '#type' => 'style_selector',
    //   '#title' => $this->t('Background Style'),
    //   '#style_type' => 'css_class',
    //   '#styles' => [
    //     'bgstyle--white' => $this->t('White'),
    //     'bgstyle--overlay' => $this->t('50% Black Overlay'),
    //     'bgstyle--red' => $this->t('Red'),
    //     'bgstyle--sunrise' => $this->t('Sunrise'),
    //     'bgstyle--squares' => $this->t('Squares'),
    //     'bgstyle--animated' => $this->t('Animated Gradient'),
    //   ],
    //   '#input' => FALSE,
    // ];

    // Sample Style Color Selector.
    // $form['sample_color_style'] = [
    //   '#type' => 'style_selector',
    //   '#title' => $this->t('Color Style'),
    //   '#style_type' => 'css_class',
    //   '#styles' => [
    //     'color--pink' => $this->t('Pink'),
    //     'color--purple' => $this->t('Purple'),
    //   ],
    //   '#input' => FALSE,
    // ];

    // $form['sample_style_large'] = [
    //   '#type' => 'style_selector',
    //   '#title' => $this->t('Sample Large Round Style Selector'),
    //   '#style_type' => 'css_class',
    //   '#ui_style' => 'round-large',
    //   '#styles' => [
    //     'bgstyle--white' => $this->t('White'),
    //     'bgstyle--red' => $this->t('Red'),
    //     'bgstyle--green' => $this->t('Green'),
    //     'bgstyle--sunrise' => $this->t('Sunrise'),
    //     'bgstyle--squares' => $this->t('Squares'),
    //     'bgstyle--animated' => $this->t('Animated Gradient'),
    //   ],
    //   '#input' => FALSE,
    // ];

    // $form['sample_style_rect'] = [
    //   '#type' => 'style_selector',
    //   '#title' => $this->t('Sample Rectangle Style Selector'),
    //   '#style_type' => 'css_class',
    //   '#ui_style' => 'rect',
    //   '#styles' => [
    //     'bgstyle--white' => $this->t('White'),
    //     'bgstyle--red' => $this->t('Red'),
    //     'bgstyle--green' => $this->t('Green'),
    //     'bgstyle--sunrise' => $this->t('Sunrise'),
    //     'bgstyle--squares' => $this->t('Squares'),
    //     'bgstyle--animated' => $this->t('Animated Gradient'),
    //   ],
    //   '#input' => FALSE,
    // ];

    // $form['sample_style_lrect'] = [
    //   '#type' => 'style_selector',
    //   '#title' => $this->t('Sample Large Rectangle Style Selector'),
    //   '#style_type' => 'css_class',
    //   '#ui_style' => 'rect-large',
    //   '#styles' => [
    //     'bgstyle--white' => $this->t('White'),
    //     'bgstyle--red' => $this->t('Red'),
    //     'bgstyle--green' => $this->t('Green'),
    //     'bgstyle--sunrise' => $this->t('Sunrise'),
    //     'bgstyle--squares' => $this->t('Squares'),
    //     'bgstyle--animated' => $this->t('Animated Gradient'),
    //   ],
    //   '#input' => FALSE,
    // ];

    // Sample Style Selector.
    // $form['sample_color'] = [
    //   '#type' => 'style_selector',
    //   '#title' => $this->t('Text Color'),
    //   '#style_type' => 'css_color',
    //   '#styles' => [
    //     '#ffffff' => $this->t('White'),
    //     '#222' => $this->t('Black'),
    //     'green' => $this->t('Green'),
    //     '#003767' => $this->t('Navy'),
    //     'rgba(0,0,0,.5)' => $this->t('Black 50%'),
    //     // '000' => $this->t('Invalid'),
    //   ],
    //   '#input' => FALSE,
    // ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // Validate the libraries.
    foreach (['shared_libraries', 'theme_libraries'] as $value) {
      $libraries = $this->textToConfigData($form_state->getValue([$value]));
      foreach ($libraries as $library) {
        // Ensure the library is defined.
        [$extension, $name] = explode('/', $library);
        if (!\Drupal::service('library.discovery')->getLibraryByName($extension, $name)) {
          $form_state->setErrorByName(
            $value,
            t('The library <strong>@lib</strong> was not found in the system.',
              [
                '@lib' => $library,
              ]
            )
          );
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Retrieve the configuration.
    $config = $this->configFactory()->getEditable(static::SETTINGS);
    // $config->set('theme_libraries', Xss::filter($form_state->getValue('theme_libraries')));
    $config->set('shared_libraries', $this->textToConfigData($form_state->getValue(['shared_libraries'])));
    $config->set('theme_libraries', $this->textToConfigData($form_state->getValue(['theme_libraries'])));
    $config->set('admin_libraries', $this->textToConfigData($form_state->getValue(['admin_libraries'])));
    $config->set('extra_css_classes', $this->classSettingToConfigData($form_state->getValue(['css_classes'])));
    $config->set('extra_color_classes', $this->classSettingToConfigData($form_state->getValue(['color_classes'])));
    $config->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Convert a multiline text setting to an array of sanitized values.
   *
   * @param string $text
   *  Text string possibly containing multiple lines.
   *
   * @return array
   *  An array of sanitized values.
   */
  private function textToConfigData(string $text) {
    $values = [];
    // Normalize newlines.
    $text = str_replace(["\r\n", "\r"], "\n", $text);
    foreach (array_filter(explode("\n", $text)) as $value) {
      $values[] = Xss::filter($value);
    }
    return $values;
  }

  /**
   * Convert a space-separated list of classes to a sanitized array suitable
   * for storage as configuration.
   *
   * @param mixed $setting
   *
   * @return array
   *  Array of valid CSS class identifiers.
   */
  private function classSettingToConfigData($setting) {
    $values = [];
    if ($setting) {
      foreach (array_filter(explode(' ', $setting)) as $value) {
        $value = Html::getClass(trim($value));
        if ($value) {
          $values[] = $value;
        }
      }
    }
    return $values;
  }

  /**
   * Convert a config array to a string suitable for editing, e.g. in a
   * textfield.
   *
   * @param $config
   *  The config item.
   *
   * @param string $separator
   *  Separator to use when imploding the array.
   *
   * @return string
   */
  private function settingStringFromConfigData($config, string $separator = ' ') {
    $setting = '';
    if ($config && is_array($config)) {
      $setting = implode($separator, $config);
    }
    return $setting;
  }

}
